@@ -1,24 +1,32 @@
 #!/usr/bin/env python
 # -*- encoding: utf-8 -*-
-
 import logging
 import re
+
 import requests
 import six
-from caldav.lib.python_utilities import to_wire, to_unicode, to_normal_str
-from lxml import etree
-
-from caldav.elements import dav, cdav, ical
-
+from caldav.elements import cdav
+from caldav.elements import dav
+from caldav.elements import ical
 from caldav.lib import error
+from caldav.lib.python_utilities import to_normal_str
+from caldav.lib.python_utilities import to_unicode
+from caldav.lib.python_utilities import to_wire
 from caldav.lib.url import URL
-from caldav.objects import Principal, Calendar, errmsg, log, ScheduleInbox, ScheduleOutbox
+from caldav.objects import Calendar
+from caldav.objects import errmsg
+from caldav.objects import log
+from caldav.objects import Principal
+from caldav.objects import ScheduleInbox
+from caldav.objects import ScheduleOutbox
+from lxml import etree
 
 if six.PY3:
     from urllib.parse import unquote
 else:
     from urlparse import unquote
 
+
 class DAVResponse:
     """
     This class is a response from a DAV request.  It is instantiated from
@@ -26,6 +34,7 @@ class DAVResponse:
     know anything about this class.  Since we often get XML responses,
     it tries to parse it into `self.tree`
     """
+
     raw = ""
     reason = ""
     tree = None
@@ -38,32 +47,44 @@ class DAVResponse:
         log.debug("response status: " + str(self.status))
 
         self._raw = response.content
-        
+
         ## TODO: this if/else/elif could possibly be refactored, or we should
         ## consider to do streaming into the xmltree library as originally
         ## intended.  It only makes sense for really huge payloads though.
-        if (self.headers.get('Content-Type', '').startswith('text/xml') or
-            self.headers.get('Content-Type', '').startswith('application/xml')):
+        if self.headers.get("Content-Type", "").startswith(
+            "text/xml"
+        ) or self.headers.get("Content-Type", "").startswith("application/xml"):
             try:
-                content_length = int(self.headers['Content-Length'])
+                content_length = int(self.headers["Content-Length"])
             except:
-                content_length=-1
-            if content_length == 0:
-                self._raw = ''
+                content_length = -1
+            if content_length == 0 or not self._raw:
+                self._raw = ""
                 self.tree = None
                 log.debug("No content delivered")
             else:
                 ## With response.raw we could be streaming the content, but it does not work because
                 ## the stream often is compressed.  We could add uncompression on the fly, but not
                 ## considered worth the effort as for now.
-                #self.tree = etree.parse(response.raw, parser=etree.XMLParser(remove_blank_text=True))
-                self.tree = etree.XML(self._raw, parser=etree.XMLParser(remove_blank_text=True))
+                # self.tree = etree.parse(response.raw, parser=etree.XMLParser(remove_blank_text=True))
+                try:
+                    self.tree = etree.XML(
+                        self._raw, parser=etree.XMLParser(remove_blank_text=True)
+                    )
+                except:
+                    logging.critical(
+                        "Expected some valid XML from the server, but got this: \n"
+                        + self._raw,
+                        exc_info=True,
+                    )
+                    raise
                 if log.level <= logging.DEBUG:
                     log.debug(etree.tostring(self.tree, pretty_print=True))
-        elif (self.headers.get('Content-Type', '').startswith('text/calendar') or
-              self.headers.get('Content-Type', '').startswith('text/plain')):
-              ## text/plain is typically for errors, we shouldn't see it on 200/207 responses.
-              ## TODO: may want to log an error if it's text/plain and 200/207.
+        elif self.headers.get("Content-Type", "").startswith(
+            "text/calendar"
+        ) or self.headers.get("Content-Type", "").startswith("text/plain"):
+            ## text/plain is typically for errors, we shouldn't see it on 200/207 responses.
+            ## TODO: may want to log an error if it's text/plain and 200/207.
             ## Logic here was moved when refactoring
             pass
         else:
@@ -74,18 +95,20 @@ class DAVResponse:
             ## TODO: maybe just remove all of the code above in this if/else and let all
             ## data be parsed through this code.
             try:
-                self.tree = etree.XML(self._raw, parser=etree.XMLParser(remove_blank_text=True))
+                self.tree = etree.XML(
+                    self._raw, parser=etree.XMLParser(remove_blank_text=True)
+                )
             except:
                 pass
 
         ## this if will always be true as for now, see other comments on streaming.
-        if hasattr(self, '_raw'):
+        if hasattr(self, "_raw"):
             log.debug(self._raw)
             # ref https://github.com/python-caldav/caldav/issues/112 stray CRs may cause problems
             if type(self._raw) == bytes:
-                self._raw = self._raw.replace(b'\r\n', b'\n')
+                self._raw = self._raw.replace(b"\r\n", b"\n")
             elif type(self._raw) == str:
-                self._raw = self._raw.replace('\r\n', '\n')
+                self._raw = self._raw.replace("\r\n", "\n")
         self.status = response.status_code
         ## ref https://github.com/python-caldav/caldav/issues/81,
         ## incidents with a response without a reason has been
@@ -93,12 +116,12 @@ class DAVResponse:
         try:
             self.reason = response.reason
         except AttributeError:
-            self.reason = ''
+            self.reason = ""
 
     @property
     def raw(self):
         ## TODO: this should not really be needed?
-        if not hasattr(self, '_raw'):
+        if not hasattr(self, "_raw"):
             self._raw = etree.tostring(self.tree, pretty_print=True)
         return self._raw
 
@@ -125,11 +148,11 @@ class DAVResponse:
         simple XPath query, but I'm not much into XPath)
         """
         tree = self.tree
-        if (tree.tag == 'xml' and tree[0].tag == dav.MultiStatus.tag):
+        if tree.tag == "xml" and tree[0].tag == dav.MultiStatus.tag:
             return tree[0]
-        if (tree.tag == dav.MultiStatus.tag):
+        if tree.tag == dav.MultiStatus.tag:
             return self.tree
-        return [ self.tree ]
+        return [self.tree]
 
     def validate_status(self, status):
         """
@@ -140,10 +163,12 @@ class DAVResponse:
         makes sense to me, but I've only seen it from SOGo, and it's
         not in accordance with the examples in rfc6578.
         """
-        if (' 200 ' not in status and
-            ' 201 ' not in status and
-            ' 207 ' not in status and
-            ' 404 ' not in status):
+        if (
+            " 200 " not in status
+            and " 201 " not in status
+            and " 207 " not in status
+            and " 404 " not in status
+        ):
             raise error.ResponseError(status)
 
     def _parse_response(self, response):
@@ -184,9 +209,9 @@ class DAVResponse:
         """
         self.objects = {}
 
-        if 'Schedule-Tag' in self.headers:
-            self.schedule_tag = self.headers['Schedule-Tag']
-        
+        if "Schedule-Tag" in self.headers:
+            self.schedule_tag = self.headers["Schedule-Tag"]
+
         responses = self._strip_to_multistatus()
         for r in responses:
             if r.tag == dav.SyncToken.tag:
@@ -196,7 +221,7 @@ class DAVResponse:
 
             (href, propstats, status) = self._parse_response(r)
             ## I would like to do this assert here ...
-            #error.assert_(not href in self.objects)
+            # error.assert_(not href in self.objects)
             ## but then there was https://github.com/python-caldav/caldav/issues/136
             if not href in self.objects:
                 self.objects[href] = {}
@@ -208,12 +233,12 @@ class DAVResponse:
                 cnt = 0
                 status = propstat.find(dav.Status.tag)
                 error.assert_(status is not None)
-                if (status is not None):
+                if status is not None:
                     error.assert_(len(status) == 0)
                     cnt += 1
                     self.validate_status(status.text)
                     ## if a prop was not found, ignore it
-                    if ' 404 ' in status.text:
+                    if " 404 " in status.text:
                         continue
                 for prop in propstat.iterfind(dav.Prop.tag):
                     cnt += 1
@@ -225,12 +250,14 @@ class DAVResponse:
 
         return self.objects
 
-    def _expand_simple_prop(self, proptag, props_found, multi_value_allowed=False, xpath=None):
+    def _expand_simple_prop(
+        self, proptag, props_found, multi_value_allowed=False, xpath=None
+    ):
         values = []
         if proptag in props_found:
             prop_xml = props_found[proptag]
             error.assert_(not prop_xml.items())
-            if not xpath and len(prop_xml)==0:
+            if not xpath and len(prop_xml) == 0:
                 if prop_xml.text:
                     values.append(prop_xml.text)
             else:
@@ -248,7 +275,7 @@ class DAVResponse:
         else:
             if not values:
                 return None
-            error.assert_(len(values)==1)
+            error.assert_(len(values) == 1)
             return values[0]
 
     ## TODO: "expand" does not feel quite right.
@@ -261,16 +288,21 @@ class DAVResponse:
         Executes find_objects_and_props if not run already, then
         modifies and returns self.objects.
         """
-        if not hasattr(self, 'objects'):
+        if not hasattr(self, "objects"):
             self.find_objects_and_props()
         for href in self.objects:
             props_found = self.objects[href]
             for prop in props:
-                props_found[prop.tag] = self._expand_simple_prop(prop.tag, props_found, xpath=xpath)
+                props_found[prop.tag] = self._expand_simple_prop(
+                    prop.tag, props_found, xpath=xpath
+                )
             for prop in multi_value_props:
-                props_found[prop.tag] = self._expand_simple_prop(prop.tag, props_found, xpath=xpath, multi_value_allowed=True)
+                props_found[prop.tag] = self._expand_simple_prop(
+                    prop.tag, props_found, xpath=xpath, multi_value_allowed=True
+                )
         return self.objects
 
+
 class DAVClient:
     """
     Basic client for webdav, uses the requests lib; gives access to
@@ -279,11 +311,20 @@ class DAVClient:
     Unless you have special needs, you should probably care most about
     the constructor (__init__), the principal method and the calendar method.
     """
+
     proxy = None
     url = None
 
-    def __init__(self, url, proxy=None, username=None, password=None,
-                 auth=None, ssl_verify_cert=True, ssl_cert=None):
+    def __init__(
+        self,
+        url,
+        proxy=None,
+        username=None,
+        password=None,
+        auth=None,
+        ssl_verify_cert=True,
+        ssl_cert=None,
+    ):
         """
         Sets up a HTTPConnection object towards the server in the url.
         Parameters:
@@ -292,6 +333,10 @@ class DAVClient:
          * username and password should be passed as arguments or in the URL
          * auth and ssl_verify_cert is passed to requests.request.
          ** ssl_verify_cert can be the path of a CA-bundle or False.
+
+        The requests library will honor a .netrc-file, if such a file exists
+        username and password may be omitted.  Known bug: .netrc is honored
+        even if a username/password is given, ref https://github.com/python-caldav/caldav/issues/206
         """
 
         self.session = requests.Session()
@@ -303,21 +348,23 @@ class DAVClient:
         if proxy is not None:
             self.proxy = proxy
             # requests library expects the proxy url to have a scheme
-            if re.match('^.*://', proxy) is None:
-                self.proxy = self.url.scheme + '://' + proxy
+            if re.match("^.*://", proxy) is None:
+                self.proxy = self.url.scheme + "://" + proxy
 
             # add a port is one is not specified
             # TODO: this will break if using basic auth and embedding
             # username:password in the proxy URL
             p = self.proxy.split(":")
             if len(p) == 2:
-                self.proxy += ':8080'
+                self.proxy += ":8080"
             log.debug("init - proxy: %s" % (self.proxy))
 
         # Build global headers
-        self.headers = {"User-Agent": "Mozilla/5.0",
-                        "Content-Type": "text/xml",
-                        "Accept": "text/xml, text/calendar"}
+        self.headers = {
+            "User-Agent": "Mozilla/5.0",
+            "Content-Type": "text/xml",
+            "Accept": "text/xml, text/calendar",
+        }
         if self.url.username is not None:
             username = unquote(self.url.username)
             password = unquote(self.url.password)
@@ -325,8 +372,8 @@ class DAVClient:
         self.username = username
         self.password = password
         ## I had problems with passwords with non-ascii letters in it ...
-        if hasattr(self.password, 'encode'):
-            self.password = self.password.encode('utf-8')
+        if hasattr(self.password, "encode"):
+            self.password = self.password.encode("utf-8")
         self.auth = auth
         # TODO: it's possible to force through a specific auth method here,
         # but no test code for this.
@@ -337,6 +384,18 @@ class DAVClient:
 
         self._principal = None
 
+    def __enter__(self):
+        return self
+
+    def __exit__(self, exc_type, exc_value, traceback):
+        self.close()
+
+    def close(self):
+        """
+        Closes the DAVClient's session object
+        """
+        self.session.close()
+
     def principal(self, *largs, **kwargs):
         """
         Convenience method, it gives a bit more object-oriented feel to
@@ -373,15 +432,15 @@ class DAVClient:
             response = self.options(self.principal().url)
         except:
             response = self.options(self.url)
-        return response.headers.get('DAV', None)
+        return response.headers.get("DAV", None)
 
     def check_cdav_support(self):
         support_list = self.check_dav_support()
-        return 'calendar-access' in support_list
+        return support_list and "calendar-access" in support_list
 
     def check_scheduling_support(self):
         support_list = self.check_dav_support()
-        return 'calendar-auto-schedule' in support_list
+        return support_list and "calendar-auto-schedule" in support_list
 
     def propfind(self, url=None, props="", depth=0):
         """
@@ -395,8 +454,7 @@ class DAVClient:
         Returns
          * DAVResponse
         """
-        return self.request(url or self.url, "PROPFIND", props,
-                            {'Depth': str(depth)})
+        return self.request(url or self.url, "PROPFIND", props, {"Depth": str(depth)})
 
     def proppatch(self, url, body, dummy=None):
         """
@@ -424,9 +482,12 @@ class DAVClient:
         Returns
          * DAVResponse
         """
-        return self.request(url, "REPORT", query,
-                            {'Depth': str(depth), "Content-Type":
-                             "application/xml; charset=\"utf-8\""})
+        return self.request(
+            url,
+            "REPORT",
+            query,
+            {"Depth": str(depth), "Content-Type": 'application/xml; charset="utf-8"'},
+        )
 
     def mkcol(self, url, body, dummy=None):
         """
@@ -482,13 +543,19 @@ class DAVClient:
         Send a delete request.
         """
         return self.request(url, "DELETE")
- 
+
     def options(self, url):
         return self.request(url, "OPTIONS")
 
-    def _pre_request(self, url, body, headers):
+    def extract_auth_types(self, header):
+        auth_types = header.lower().split(",")
+        auth_types = map(lambda auth_type: auth_type.strip(), auth_types)
+        auth_types = map(lambda auth_type: auth_type[: auth_type.find(" ")], auth_types)
+        return list(filter(lambda auth_type: auth_type, auth_types))
+
+    def request(self, url, method="GET", body="", headers={}):
         """
-        refacored out some stuff from requests, to be reused in verify_login
+        Actually sends the request
         """
         combined_headers = self.headers.copy()
         combined_headers.update(headers)
@@ -500,127 +567,105 @@ class DAVClient:
             proxies = {url.scheme: self.proxy}
             log.debug("using proxy - %s" % (proxies))
 
-        return (combined_headers, proxies)
-
-    def verify_login(self, url=None, method="PROPFIND", body="", headers={}):
-        """
-        Will do the following:
-        * run a test request without auth towards the server.
-        * assert it returns 401
-        * read the WWW-Authenticate header and decide what kind of auth object to create
-        * run a test query with auth
-        * assert it returns 2xx or 3xx
-        In 0.9, it should return True or raise an exception
-        In 0.8.2, it may log an error and return False
-        """
-        if not url:
-            url=self.url
-
-        (combined_headers, proxies) = self._pre_request(url, body, headers)
-
-        if not self.auth:
-            ## Try a test request w/o auth
-            resp = self.session.request(
-                method, url, data=to_wire(body),
-                headers=combined_headers, proxies=proxies,
-                verify=self.ssl_verify_cert, cert=self.ssl_cert)
-
-            if resp.status_code > 399 and not self.password and not self.username:
-                return True
-
-            ## I'd like to raise an AuthorizationError here, but "assert_" and
-            ## return is safer for a minor release.
-            error.assert_(resp.status_code == 401)
-            error.assert_('WWW-Authenticate' in resp.headers)
-            if resp.status_code != 401:
-                return True
-            if not 'WWW-Authenticate' in resp.headers:
-                return False
-
-            auth_type = resp.headers['WWW-Authenticate']
-            auth_type = auth_type[0:auth_type.find(" ")]
-
-            error.assert_(auth_type in ('Basic', 'Digest'))
-
-            if auth_type == 'Basic':
-                self.auth = requests.auth.HTTPBasicAuth(self.username, self.password)
-            elif auth_type == 'Digest':
-                self.auth = requests.auth.HTTPDigestAuth(self.username, self.password)
-            else:
-                ## I'm a bit concerned ... don't want to raise new exceptions in a minor release
-                #raise NotImplementedError(f"Auth method {auth_type} not supported yet")
-                return False
-
-        resp = self.session.request(
-            method, url, data=to_wire(body),
-            headers=combined_headers, proxies=proxies, auth=self.auth,
-            verify=self.ssl_verify_cert, cert=self.ssl_cert)
-
-        if resp.status_code > 399:
-            raise error.AuthorizationError(url=url, reason=resp.reason)
-
-    def request(self, url, method="GET", body="", headers={}):
-        """
-        Actually sends the request
-        """
-        (combined_headers, proxies) = self._pre_request(url, body, headers)
-
         # objectify the url
         url = URL.objectify(url)
 
-        # ensure that url is a normal string
-        url = str(url)
-
-        auth = None
-
-        if self.auth is None:
-            self.verify_login()
-
-        if self.auth is None and self.username is not None:
-            auth = requests.auth.HTTPDigestAuth(self.username, self.password)
-        else:
-            auth = self.auth
-
         log.debug(
-            "sending request - method={0}, url={1}, headers={2}\nbody:\n{3}"
-            .format(method, url, combined_headers, to_normal_str(body)))
+            "sending request - method={0}, url={1}, headers={2}\nbody:\n{3}".format(
+                method, str(url), combined_headers, to_normal_str(body)
+            )
+        )
 
-        r = self.session.request(
-            method, url, data=to_wire(body),
-            headers=combined_headers, proxies=proxies, auth=auth,
-            verify=self.ssl_verify_cert, cert=self.ssl_cert, stream=False) ## TODO: optimize with stream=True maybe
-        log.debug("server responded with %i %s" % (r.status_code, r.reason))
-        response = DAVResponse(r)
-
-
-        # If server supports BasicAuth and not DigestAuth, let's try again:
-        if response.status == 401 and self.auth is None and auth is not None:
-            auth = requests.auth.HTTPBasicAuth(self.username, self.password)
-            r = self.session.request(method, url, data=to_wire(body),
-                                 headers=combined_headers, proxies=proxies,
-                                 auth=auth, verify=self.ssl_verify_cert, cert=self.ssl_cert)
+        try:
+            r = self.session.request(
+                method,
+                str(url),
+                data=to_wire(body),
+                headers=combined_headers,
+                proxies=proxies,
+                auth=self.auth,
+                verify=self.ssl_verify_cert,
+                cert=self.ssl_cert,
+            )
+            log.debug("server responded with %i %s" % (r.status_code, r.reason))
             response = DAVResponse(r)
+        except:
+            ## this is a workaround needed due to some weird server
+            ## that would just abort the connection rather than send a
+            ## 401 when an unauthenticated request with a body was
+            ## sent to the server - ref https://github.com/python-caldav/caldav/issues/158
+            if self.auth or not self.password:
+                raise
+            r = self.session.request(
+                method="GET",
+                url=str(url),
+                headers=combined_headers,
+                proxies=proxies,
+                verify=self.ssl_verify_cert,
+                cert=self.ssl_cert,
+            )
+            if not r.status_code == 401:
+                raise
 
-        self.auth = auth
+        if (
+            r.status_code == 401
+            and "WWW-Authenticate" in r.headers
+            and self.password
+            and self.username
+            and not self.auth
+        ):
+            auth_types = self.extract_auth_types(r.headers["WWW-Authenticate"])
 
-        # this is an error condition the application wants to know
-        if response.status == requests.codes.forbidden or \
-                response.status == requests.codes.unauthorized:
-            ex = error.AuthorizationError()
-            ex.url = url
-            ## ref https://github.com/python-caldav/caldav/issues/81,
-            ## incidents with a response without a reason has been
-            ## observed
+            if "basic" in auth_types:
+                self.auth = requests.auth.HTTPBasicAuth(self.username, self.password)
+            elif "digest" in auth_types:
+                self.auth = requests.auth.HTTPDigestAuth(self.username, self.password)
+            else:
+                raise NotImplementedError(
+                    "The server does not provide any of the currently "
+                    "supported authentication methods: basic, digest"
+                )
+
+            return self.request(url, method, body, headers)
+
+        elif (
+            r.status_code == 401
+            and "WWW-Authenticate" in r.headers
+            and self.password
+            and self.username
+            and self.auth
+        ):
+
+            ## Some (ancient) servers don't like UTF-8 binary auth with Digest authentication.
+            ## An example are old SabreDAV based servers.  Not sure about UTF-8 and Basic Auth,
+            ## but likely the same.  so retry if password is a bytes sequence and not a string
+            ## (see commit 13a4714, which introduced this regression)
+
+            auth_types = self.extract_auth_types(r.headers["WWW-Authenticate"])
+
+            if hasattr(self.password, "decode"):
+                if "basic" in auth_types:
+                    self.auth = requests.auth.HTTPBasicAuth(
+                        self.username, self.password.decode()
+                    )
+                elif "digest" in auth_types:
+                    self.auth = requests.auth.HTTPDigestAuth(
+                        self.username, self.password.decode()
+                    )
+
+            self.username = None
+            self.password = None
+            return self.request(url, method, body, headers)
+
+        # this is an error condition that should be raised to the application
+        if (
+            response.status == requests.codes.forbidden
+            or response.status == requests.codes.unauthorized
+        ):
             try:
-                ex.reason = response.reason
+                reason = response.reason
             except AttributeError:
-                ex.reason = "None given"
-            raise ex
-
-        # let's save the auth object and remove the user/pass information
-        if not self.auth and auth:
-            self.auth = auth
-            del self.username
-            del self.password
+                reason = "None given"
+            raise error.AuthorizationError(url=str(url), reason=reason)
 
         return response
